package it.bb.wof.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.SessionContext;
import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import it.bb.wof.model.Denominazione;
import it.bb.wof.model.Regione;

/**
 * Backing bean for Denominazione entities.
 * <p>
 * This class provides CRUD functionality for all Denominazione entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class DenominazioneBean implements Serializable
{

   private static final long serialVersionUID = 1L;

   /*
    * Support creating and retrieving Denominazione entities
    */

   private Long id;

   public Long getId()
   {
      return this.id;
   }

   public void setId(Long id)
   {
      this.id = id;
   }

   private Denominazione denominazione;

   public Denominazione getDenominazione()
   {
      return this.denominazione;
   }

   @Inject
   private Conversation conversation;

   @PersistenceContext(type = PersistenceContextType.EXTENDED)
   private EntityManager entityManager;

   public String create()
   {

      this.conversation.begin();
      return "create?faces-redirect=true";
   }

   public void retrieve()
   {

      if (FacesContext.getCurrentInstance().isPostback())
      {
         return;
      }

      if (this.conversation.isTransient())
      {
         this.conversation.begin();
      }

      if (this.id == null)
      {
         this.denominazione = this.example;
      }
      else
      {
         this.denominazione = findById(getId());
      }
   }

   public Denominazione findById(Long id)
   {

      return this.entityManager.find(Denominazione.class, id);
   }

   /*
    * Support updating and deleting Denominazione entities
    */

   public String update()
   {
      this.conversation.end();

      try
      {
         if (this.id == null)
         {
            this.entityManager.persist(this.denominazione);
            return "search?faces-redirect=true";
         }
         else
         {
            this.entityManager.merge(this.denominazione);
            return "view?faces-redirect=true&id=" + this.denominazione.getIdDenominazione();
         }
      }
      catch (Exception e)
      {
         FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(e.getMessage()));
         return null;
      }
   }

   public String delete()
   {
      this.conversation.end();

      try
      {
         Denominazione deletableEntity = findById(getId());

         this.entityManager.remove(deletableEntity);
         this.entityManager.flush();
         return "search?faces-redirect=true";
      }
      catch (Exception e)
      {
         FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(e.getMessage()));
         return null;
      }
   }

   /*
    * Support searching Denominazione entities with pagination
    */

   private int page;
   private long count;
   private List<Denominazione> pageItems;

   private Denominazione example = new Denominazione();

   public int getPage()
   {
      return this.page;
   }

   public void setPage(int page)
   {
      this.page = page;
   }

   public int getPageSize()
   {
      return 10;
   }

   public Denominazione getExample()
   {
      return this.example;
   }

   public void setExample(Denominazione example)
   {
      this.example = example;
   }

   public void search()
   {
      this.page = 0;
   }

   public void paginate()
   {

      CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

      // Populate this.count

      CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
      Root<Denominazione> root = countCriteria.from(Denominazione.class);
      countCriteria = countCriteria.select(builder.count(root)).where(
            getSearchPredicates(root));
      this.count = this.entityManager.createQuery(countCriteria)
            .getSingleResult();

      // Populate this.pageItems

      CriteriaQuery<Denominazione> criteria = builder.createQuery(Denominazione.class);
      root = criteria.from(Denominazione.class);
      TypedQuery<Denominazione> query = this.entityManager.createQuery(criteria
            .select(root).where(getSearchPredicates(root)));
      query.setFirstResult(this.page * getPageSize()).setMaxResults(
            getPageSize());
      this.pageItems = query.getResultList();
   }

   private Predicate[] getSearchPredicates(Root<Denominazione> root)
   {

      CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
      List<Predicate> predicatesList = new ArrayList<Predicate>();

      String codiceDenominazione = this.example.getCodiceDenominazione();
      if (codiceDenominazione != null && !"".equals(codiceDenominazione))
      {
         predicatesList.add(builder.like(root.<String> get("codiceDenominazione"), '%' + codiceDenominazione + '%'));
      }
      Regione regione = this.example.getRegione();
      if (regione != null)
      {
         predicatesList.add(builder.equal(root.get("regione"), regione));
      }

      return predicatesList.toArray(new Predicate[predicatesList.size()]);
   }

   public List<Denominazione> getPageItems()
   {
      return this.pageItems;
   }

   public long getCount()
   {
      return this.count;
   }

   /*
    * Support listing and POSTing back Denominazione entities (e.g. from inside an
    * HtmlSelectOneMenu)
    */

   public List<Denominazione> getAll()
   {

      CriteriaQuery<Denominazione> criteria = this.entityManager
            .getCriteriaBuilder().createQuery(Denominazione.class);
      return this.entityManager.createQuery(
            criteria.select(criteria.from(Denominazione.class))).getResultList();
   }

   @Resource
   private SessionContext sessionContext;

   public Converter getConverter()
   {

      final DenominazioneBean ejbProxy = this.sessionContext.getBusinessObject(DenominazioneBean.class);

      return new Converter()
      {

         @Override
         public Object getAsObject(FacesContext context,
               UIComponent component, String value)
         {

            return ejbProxy.findById(Long.valueOf(value));
         }

         @Override
         public String getAsString(FacesContext context,
               UIComponent component, Object value)
         {

            if (value == null)
            {
               return "";
            }

            return String.valueOf(((Denominazione) value).getIdDenominazione());
         }
      };
   }

   /*
    * Support adding children to bidirectional, one-to-many tables
    */

   private Denominazione add = new Denominazione();

   public Denominazione getAdd()
   {
      return this.add;
   }

   public Denominazione getAdded()
   {
      Denominazione added = this.add;
      this.add = new Denominazione();
      return added;
   }
}